DAT programName         byte "SayItBridgeAndMonitor130305a", 0
CON
{{
  ******* Public Notes *******
  This program displays (in a terminal
  window) the data exchange between a
  SayIt Module and the PC GUI program.

  An additional serial line is required
  since the GUI will have control of the
  Prop's normal serial connection. The
  additional serial line may be provided
  by a Prop Plug or equivelent.

  This program may be used as a bridge
  between the PC and SayIt without
  using the additional serial line but
  you wont be able to monitor the
  exchange between PC and SayIt.

  There are two related SayIt programs
  available on the Parallx Propeller
  Forums.
  SayItSimpleBridge.spin provides a
  bridge between PC and SayIt without
  having the option to monitor the
  exchange between the PC and SayIt.
  There really isn't an advantage to
  using the SayItSimpleBridge program
  over using this program.
  The other useful object is
  SayItAlpha1305a.spin. SayItAlpha
  demonstates how to use the various
  word sets to select robot actions.

  The program just mentioned may be
  found in posts #15 and #19 of this
  thread:
  
  http://forums.parallax.com/showthread.php/120475

  I plan to update the SayItAlpha object
  to include speaker dependent word.
  
  
}}
{
  ******* Private Notes *******

  SayItBridgeAndMonitor130304a
  Modified from SayItAlpha110609b.spin
  130305a Use multi port serial object.
  Remove EEPROM options and direct
  control of SayIt from terminal.
   

}
{{  Copied from "Say It Demo.bs2"
' Protocol Command
CMD_BREAK           CON         "b" ' abort recog or ping
CMD_SLEEP           CON         "s" ' go to power down
CMD_KNOB            CON         "k" ' set si knob <1>
CMD_LEVEL           CON         "v" ' set sd level <1>
CMD_LANGUAGE        CON         "l" ' set si language <1>
CMD_TIMEOUT         CON         "o" ' set timeout <1>
CMD_RECOG_SI        CON         "i" ' do si recog from ws <1>
CMD_RECOG_SD        CON         "d" ' do sd recog at group <1> (0 = trigger mixed si/sd)

' Protocol Status
STS_AWAKEN          CON         "w" ' back from power down mode
STS_ERROR           CON         "e" ' signal error code <1-2>
STS_INVALID         CON         "v" ' invalid command or argument
STS_TIMEOUT         CON         "t" ' timeout expired
STS_INTERR          CON         "i" ' back from aborted recognition (see 'break')
STS_SUCCESS         CON         "o" ' no errors status
STS_RESULT          CON         "r" ' recognised sd command <1> - training similar to sd <1>
STS_SIMILAR         CON         "s" ' recognised si <1> (in mixed si/sd) - training similar to si <1>

' Protocol arguments are in the range 0x40 (-1) TO 0x60 (+31) inclusive
ARG_MIN             CON         64 ' 0x40
ARG_MAX             CON         96 ' 0x60
ARG_ZERO            CON         65 ' 0x41

ARG_ACK             CON         32 ' 0x20    'TO READ more status arguments

' Wordset
WST                 CON         0  ' wordset trigger
WS1                 CON         1  ' Wordset 1 commands
WS2                 CON         2  ' Wordset 2 actions
WS3                 CON         3  ' Wordset 3 numbers

'Wordset Commands
WS1_Action          CON   0
WS1_Move            CON   1
WS1_Turn            CON   2
WS1_Run             CON   3
WS1_Look            CON   4
WS1_Attack          CON   5
WS1_Stop            CON   6
WS1_Hello           CON   7

WS2_Left            CON   0
WS2_Right           CON   1
WS2_Up              CON   2
WS2_Down            CON   3
WS2_Forward         CON   4
WS2_Backward        CON   5

WS3_Zero            CON   0
WS3_One             CON   1
WS3_Two             CON   2
WS3_Three           CON   3
WS3_Four            CON   4
WS3_Five            CON   5
WS3_Six             CON   6
WS3_Seven           CON   7
WS3_Eight           CON   8
WS3_Nine            CON   9
WS3_Ten             CON   10

WS_Timeout          CON   254
WS_Error            CON   255

VAR_LANG            DATA 0
VAR_KNOB            DATA 2
VAR_LEVEL           DATA 2


'Robot Constant
COMMAND_DURATION    CON   2000
COMMAND_SPEED_HIGH  CON   3
COMMAND_SPEED_LOW   CON   1

'Global Variable
counter             VAR   Word
VRA                 VAR   Byte
VRA1                VAR   Byte
VRLED               VAR   Byte
WS                  VAR   Byte
RXC                 VAR   Byte
RXC_PREV            VAR   Byte
}}
CON

  ' Copied from "Say It Demo.bs2"
  ' Protocol Command
  CMD_BREAK = "b" ' abort recog or ping
  CMD_SLEEP = "s" ' go to power down
  CMD_KNOB = "k" ' set si knob <1>
  CMD_LEVEL = "v" ' set sd level <1>
  CMD_LANGUAGE = "l" ' set si language <1>
  CMD_TIMEOUT = "o" ' set timeout <1>
  CMD_RECOG_SI = "i" ' do si recog from ws <1>
  CMD_RECOG_SD = "d" ' do sd recog at group <1> (0 = trigger mixed si/sd)
  CMD_ID = "x"
  CMD_DUMP_SD = "p"
  CMD_MASK_SD = "m"
  CMD_COUNT_SD = "c"
  
  ' Protocol Status
  STS_AWAKEN = "w" ' back from power down mode
  STS_ERROR = "e" ' signal error code <1-2>
  STS_INVALID = "v" ' invalid command or argument
  STS_TIMEOUT = "t" ' timeout expired
  STS_INTERR = "i" ' back from aborted recognition (see 'break')
  STS_SUCCESS = "o" ' no errors status
  STS_RESULT = "r" ' recognised sd command <1> - training similar to sd <1>
  STS_SIMILAR = "s" ' recognised si <1> (in mixed si/sd) - training similar to si <1>
 
  ' Protocol arguments are in the range 0x40 (-1) TO 0x60 (+31) inclusive
  ARG_MIN = 64 ' 0x40
  ARG_MAX = 96 ' 0x60
  ARG_ZERO = 65 ' 0x41
   
  ARG_ACK = 32 ' 0x20    'TO READ more status arguments
   
  ' Wordset
  WST = 0  ' wordset trigger
  WS1 = 1  ' Wordset 1 commands
  WS2 = 2  ' Wordset 2 actions
  WS3 = 3  ' Wordset 3 numbers
   
  'Wordset Commands
  WS1_Action = 0
  WS1_Move = 1
  WS1_Turn = 2
  WS1_Run = 3
  WS1_Look = 4
  WS1_Attack = 5
  WS1_Stop = 6
  WS1_Hello = 7
   
  WS2_Left = 0
  WS2_Right = 1
  WS2_Up = 2
  WS2_Down = 3
  WS2_Forward = 4
  WS2_Backward = 5
   
  WS3_Zero = 0
  WS3_One = 1
  WS3_Two = 2
  WS3_Three = 3
  WS3_Four = 4
  WS3_Five = 5
  WS3_Six = 6
  WS3_Seven = 7
  WS3_Eight = 8
  WS3_Nine = 9
  WS3_Ten = 10
   
  WS_Timeout = 254
  WS_Error = 255
{
VAR_LANG            DATA 0
VAR_KNOB            DATA 2
VAR_LEVEL           DATA 2
}

  'Robot Constant
  COMMAND_DURATION = 2000
  COMMAND_SPEED_HIGH = 3
  COMMAND_SPEED_LOW = 1

  ' language enumeration
  #0, _English, _Italian, _Japanese, _German  
   
CON

  _Clkmode = xtal1 + pll16x     ' 80MHz  Change these two settings to match
  _Xinfreq = 5_000_000          ' your setup.


  ' Pin Assignments
  _SayItLed = 16                  ' Connected to Led on Say It

  _BridgeTxPin = 18 
  _BridgeRxPin = 19 
  
  _SayItTx = 27                 ' Connected to Rx on Say It
  _SayItRx = 26                 ' Connected to Tx on Say It 
  _DebugTxPin = 30
  _DebugRxPin = 31

  
  _SayItBaud = 9600
  BRIDGE_BAUD = _SayItBaud
  _DebugBaud =  115200

  _LargeBufferSize = 32
  _WordsetLabelSize = 9
  _DefaultWakeAttempts = 8
  _WordsetAttempts = 8
  _TxDelay = 80_000
  _ISetSize0 = 1
  _ISetSize1 = 8
  _ISetSize2 = 6
  _ISetSize3 = 11
 
  
  ' communication enumeration
  #0, _SayItCom, _DebugCom, _BridgeCom

  ' mode enumeration
  #0, _ListenForWordSet0, _ListenForWordSet1, _ListenForWordSet2, {
    } _ListenForWordSet3, _NotListening, _ListenForWordSet3Hundreds,  _ListenForWordSet3tens, {
    } _ListenForWordSet3units, _TrainSDGroup0 
    
  ' word set enumeration
  #0, _TiggerWs0, _TypeWs1, _DirectionWs2, _AmountWs3, _NotListeningWs4

  ' wordset1 enumeration
  #0, _ActionWs1, _MoveWs1, _TurnWs1, _RunWs1, _LookWs1, _AttackWs1, _StopWs1, _HelloWs1

  ' wordset2 enumeration
  #0, _LeftWs2, _RightWs2, _UpWs2, _DownWs2, _ForwardWs2, _BackwardWs2

  ' wordset3 enumeration
  #0, _ZeroWs3, _OneWs3, _TwoWs3, _ThreeWs3, _FourWs3, _FiveWs3, _SixWs3, _SevenWs3, _EightWs3, _NineWs3, _TenWs3

  ' next action enumeration
  #0, _NoActionNext, _ActionNext, _MoveLeftNext, _MoveRightNext, _MoveUpNext, _MoveDownNext, {
    } _MoveForwardNext, _MoveBackwardNext, _TurnLeftNext, _TurnRightNext, _TurnUpNext, {
    } _TurnDownNext, _TurnForwardNext, _TurnBackwardNext, _RunNext, _LookLeftNext, {
    } _LookRightNext, _LookUpNext, _LookDownNext, _LookForwardNext, _LookBackwardNext, {
    } _AttackLeftNext, _AttackRightNext, _AttackUpNext, _AttackDownNext, {
    } _AttackForwardNext, _AttackBackwardNext, _StopNext, _HelloNext

VAR
 
  long cog
  
  word globalCounter

  byte vra, vra1, vrLed, ws, rxc, rxcPrevious
  byte globalMode, previousGlobalMode
  byte previousIndex, priorPreviousIndex, nextAction 
  byte similarFlag 
  
OBJ

  Com : "pcFullDuplexSerial4FC128"                 ' uses one cog
  Eeprom : "Propeller EepromDWD"
  
PUB Main | localIndex, localAttmpts

  globalMode := _NotListening 
  nextAction := _NoActionNext
               
  outa[_SayItLed]~  
  dira[_SayItLed]~~  
 
  Com.AddPort(_SayItCom, _SayItRx, _SayItTx, -1, -1, 0, 0, _SayItBaud)  
  Com.AddPort(_DebugCom, _DebugRxPin, _DebugTxPin, -1, -1, 0, 0, _DebugBaud)
  Com.AddPort(_BridgeCom, _BridgeRxPin, _BridgeTxPin, -1, -1, 0, 0, _SayItBaud)
           
  Com.Start
 
  Bridge


PUB Bridge | localCharacter, currentSource, previousSource

  currentSource := -1
  previousSource := -1
  
  Com.Str(_DebugCom,  string(13, "Entering bridge mode."))
                       
  repeat
    localCharacter := Com.rxcheck(_SayItCom)
    if localCharacter <> -1 and localCharacter <> 0
      
      currentSource := _SayItCom
      Com.tx(_BridgeCom, localCharacter)
      
      if previousSource <> currentSource
        outa[_SayItLed] := 1
        previousSource := currentSource
        Com.Str(_DebugCom,  string(13, "Say It: "))
      TbugTx(localCharacter)
      
    localCharacter := Com.rxcheck(_BridgeCom)
    if localCharacter <> -1 and localCharacter <> 0
      
      currentSource := _BridgeCom
      Com.tx(_SayItCom, localCharacter) 

      if previousSource <> currentSource
        outa[_SayItLed] := 0
        previousSource := currentSource
        Com.Str(_DebugCom,  string(13, "PC: "))
      TbugTx(localCharacter)

PRI TbugTx(localCharacter)

  if (localCharacter > 32 and localCharacter < 127) or localCharacter == 13
    Com.tx(_DebugCom, localCharacter)
  else
    Com.tx(_DebugCom, 60)
    Com.tx(_DebugCom, 36)
    TbugHex(localCharacter, 2)
    Com.tx(_DebugCom, 62)
  
    
PRI TbugHex(localHex, localSize)
 
  Com.hex(_DebugCom, localHex, localSize)